/**	    code: for  Dungey and Martin "Unravelling Financial Market Linkages During Crises", Journal of Applied Econometrics, forthcoming

***     Indirect estimates of the conditional Factor model with garch (world and two market factors)
***     of exchange rates in the East Asian Crisis
***
***     Data are daily from 2/1/95 to 31/8/98
***
***     Order of variables is:
***
***       
***			KOREA
***			INDONESIA
***			THAILAND
***			MALAYSIA
***			US
***			AUSTRALIA
***
***     Complete model
***
***
***		markets are ordered: Currency then Equity
***
***
***			b denotes bonds, c denotes currencies, e denotes equities
***
**/

library optmum, pgraph,maxlik;
#include optmum.ext;
#include maxlik.ext;
optset;
gausset;
graphset;
maxset;


format 12,5;
/*output file = c:\gmm.out reset;*/

rndseed 123457;

@********************************DATA *****************************************@



load rt[956,5]=c:\diskette\mardi\multi\fxdata2.txt;	@exchange rate data - Korea, Indonesia, Thailand, Malaysia, Australia@

erate=ln(rt);
erate=100*(trimr(erate,1,0)-trimr(erate,0,1));

erate=erate[652:955,.];


y1t = erate - meanc(erate)';


/**		Filter ar(1) from Korea		**/

y = trimr(y1t[.,1],1,0);
x = trimr(y1t[.,1],0,1);

y = y - (y/x)*x;

y1t = y ~ trimr(y1t[.,2:cols(y1t)],1,0);


load rt1[956,7]=c:\diskette\mardi\multi\sdata.txt;		@equity data@
stock=ln(rt1);
stock=100*(trimr(stock,1,0)-trimr(stock,0,1));
stock=stock[.,6 2 1 4 3 7];   @US, Korea, Indonesia, Thailand, Malaysia, Australia@
stock=stock[652:955,.];
y3t = stock - meanc(stock)';
y3t=trimr(y3t,1,0);


tt = seqa(1997,1/260,303);


yt = y1t~y3t;   /**		currency and equities    **/



@------------------------------------------------------------------------------------------------------------------------------------------
				Calculate the auxiliary model of the data
--------------------------------------------------------------------------------------------------------------------------------------------@


itype = 0.0;


	{va,ve} = eigrs2(corrx(yt));

	loadings = ve[.,(cols(ve)-2):cols(ve)] ; @---  choose best three principal components   ---@


	ytfact = yt*loadings ;  

	y = trimr(yt,1,0);
	x = trimr(ytfact,0,1);
	bhat1 = y/x;
	u1 = y - x*bhat1;
	g1 = u1 *~ x;


y0 = (u1-meanc(u1)') *~ (u1-meanc(u1)');		@cov matrix calc as 1^2,1*2,1*3,1*4,2*1,2^2,2*3,2*4.....4^4@
y0   = y0[.,vech( reshape(seqa(1,1,cols(u1)^2),cols(u1),cols(u1)) )] ;		@take the lower diagonal of y2@
bhat0 = meanc(y0);
g0 = y0 - bhat0';	


	
	pc1 = ytfact[.,3];
	dpc1 = trimr(pc1,1,0) - trimr(pc1,0,1);

	
	pc2 = ytfact[.,2];
	dpc2 = trimr(pc2,1,0) - trimr(pc2,0,1);

	
	pc3 = ytfact[.,1];
	dpc3 = trimr(pc3,1,0) - trimr(pc3,0,1);


	zt = (dpc1.^2 - meanc(dpc1.^2))/stdc(dpc1.^2);
	z2 = trimr(zt,2,0).*trimr(zt,1,1) ~ trimr(zt,2,0).*trimr(zt,0,2);
	bhat2 = meanc(z2);
	g2 = z2 - bhat2';



	zt = (dpc2.^2 - meanc(dpc2.^2))/stdc(dpc2.^2);
	z2 = trimr(zt,2,0).*trimr(zt,1,1) ~ trimr(zt,2,0).*trimr(zt,0,2);
	bhat3 = meanc(z2);
	g3 = z2 - bhat3';


	zt = (dpc3.^2 - meanc(dpc3.^2))/stdc(dpc3.^2);
	z2 = trimr(zt,2,0).*trimr(zt,1,1) ~ trimr(zt,2,0).*trimr(zt,0,2);
	bhat4 = meanc(z2);
	g4 = z2 - bhat4';

    

bhat =  vec(bhat1) | vec(bhat0)  |  bhat2 | bhat3 | bhat4;

g = trimr( g1 ~ g0 , 2, 0)   ~ g2 ~ g3 ~ g4;






m = cols(g);
i0  = g'g;

    

lmax = 5.0;
l=1;
    do until l>lmax;
       wtau = g[(l+1):rows(g),.]'g[1:(rows(g)-l),.];			@the squares and cross products@
       i0   = i0 + (1.0-l/(lmax+1))*(wtau + wtau');			@weighted sum of the squares and cross products@	
       l    = l + 1;		
    endo;

i0 = i0/rows(g);									@take the average@

sigma = inv( i0 );								@take the inverse@
nobs=rows(yt);



/*      Parameters for indirect estimation                  */
    t   = nobs;            /*  Sample size                  */
    h   = 100;               /*  Number of simulation paths   */
    n   = 100+h*t;               /*  Length of simulated series   */


	
@exchange market effects@

@country and market specific effect@
	ac0=1;
	ac1=1;
	ac2=1;
	ac3=1;
	ac4=1;
	ac5=1;

@exchange market effect@
	bc1=1;
	bc2=1;
	bc3=1;
	bc4=1;
	bc5=1;

@country effect@
	cc0=1;
	cc1=1;
	cc2=1;
	cc3=1;
	cc4=1;
	cc5=1;

@world effect@
	dc1=1;
	dc2=1;
	dc3=1;
	dc4=1;
	dc5=1;

@ parameters on equity markets@
@country and market specific effect@
	ae0=1;
	ae1=1;
	ae2=1;
	ae3=1;
	ae4=1;
	ae5=1;

@equity market effect@
	be0=1;
	be1=1;
	be2=1;
	be3=1;
	be4=1;
	be5=1;

@country effect@
	ce0=1;
	ce1=1;
	ce2=1;
	ce3=1;
	ce4=1;
	ce5=1;

@world effect@
	de0=1;
	de1=1;
	de2=1;
	de3=1;
	de4=1;
	de5=1;

@market spillovers - from each market to the other@
@currency market@
	@spillover to currency from equity@
	fce1=1;
	fce2=1;
	fce3=1;
	fce4=1;
	fce5=1;
@equity market@
	@spillover to equity from currency@ 
	fec0=1;
	fec1=1;
	fec2=1;
	fec3=1;
	fec4=1;
	fec5=1;

@time zone spillovers adjustments for equity markets@
	dm1=0.01;
	dm2=0.01;
	dm3=0.01;
	dm4=0.01;
	dm5=0.01;

@contagion@
	@contagion to equity from currency@ 
	gec0=1;
	gec1=1;
	gec2=1;
	gec3=1;
	gec4=1;
	gec5=1;
	@contagion to currency from equity@
	gce1=1;
	gce2=1;
	gce3=1;
	gce4=1;
	gce5=1;

@AR structure@
	@AR coefficient on the country factors@	
	pc0=0;
	pc1=0;
	pc2=0;
	pc3=0;
	pc4=0;
	pc5=0;



	@AR coefficients on the two market factors and the world factor@
	pmc=0;
	pme=0;
	pmw=0;	


alphamc = 0.0;
betamc   = 0.0;

alphame = 0.0;
betame   = 0.0;

alphaw = 0.0;
betaw   = 0.0;


m=zeros(66,1);
mt=zeros(t,66);

            ewt = zeros(n,1);
            e0t = zeros(n,1);
            e1t = zeros(n,1);
            e2t = zeros(n,1);
            e3t = zeros(n,1);
            e4t = zeros(n,1);

	h0 = 0.1; 

            hwt = h0*ones(n,1);
            h0t = h0*ones(n,1);
            h1t = h0*ones(n,1);
            h2t = h0*ones(n,1);
            h3t = h0*ones(n,1);
            h4t = h0*ones(n,1);
  

            fwt = zeros(n,1);
            f0t = zeros(n,1);
            f1t = zeros(n,1);
            f2t = zeros(n,1);
            f3t = zeros(n,1);
            f4t = zeros(n,1);


	hmct = zeros(n,1);
	hmet = zeros(n,1);
	hwt = zeros(n,1);

	emct = zeros(n,1);	
	emet = zeros(n,1);
	ewt   = zeros(n,1);

	emetl =  zeros(n,1);
	emctl =  zeros(n,1);

	mct=zeros(n+1,1);
	met=zeros(n+1,1);
	wt=zeros(n+1,1);

	mctl=zeros(n,1);
	metl=zeros(n,1);

/*      Generate errors to simulate the true model      */
    uwt = rndn(n,1);      	/** world factor  **/

    umct = rndn(n+1,1);		/** currency market factor  **/
/*    umctl = trimr(umct,0,1);
    umct = trimr(umct,1,0);    */

    umet = rndn(n+1,1);		/** equity market factor  **/
/*    umetl = trimr(umet,0,1);
    umet = trimr(umet,1,0);    */
    


    uz0t = rndn(n,1);		/** country numeraire factor  **/
    uz1t = rndn(n,1);		/** country 1 factor   **/
    uz2t = rndn(n,1);
    uz3t = rndn(n,1);
    uz4t = rndn(n,1);
    uz5t = rndn(n,1);		/** country 5 factor   **/


    ucc0t = rndn(n,1);		/** idiosyncratic for country 1 and currency market   **/
    ucc1t = rndn(n,1);		/** idiosyncratic for country 2 and currency market    **/
    ucc2t = rndn(n,1);
    ucc3t = rndn(n,1);
    ucc4t = rndn(n,1);
    ucc5t = rndn(n,1);		/** idiosyncratic for country 5 and currency market   **/

    uce0t = rndn(n+1,1);		/** idiosyncratic for country numeraire and equity market   **/
    uce0tlg=trimr(uce0t,0,1);
    uce0t=trimr(uce0t,1,0);
    uce1t = rndn(n,1);		/** idiosyncratic for country 1 and equity market                   **/
    uce2t = rndn(n,1);
    uce3t = rndn(n,1);
    uce4t = rndn(n,1);
    uce5t = rndn(n,1);		/** idiosyncratic for country 5 and equity market                      **/




   uT=rndn(n,1);		@errors for Thailand@

    ytsim = 0.0;




/*      The objective function to be minimized          */
    proc mbar(b);
        local i,k,dq,ftsim,htsim,etsim,a0,b0,r0,ytsimt;

        dq = zeros(1,rows(vec(bhat)));			@h is the number of simulations @

        /*  Simulate the true model     */

  	ac0=b[1];
	ac1=b[2];
	ac2=b[3];
	ac3=b[4];
	ac4=b[5];
	ac5=b[6];

	bc1=b[7];
	bc2=b[8];
	bc3=b[9];
	bc4=b[10];
	bc5=b[11];

	cc0=b[12];
	cc1=b[13];
	cc2=b[14];
	cc3=b[15];
	cc4=b[16];
	cc5=b[17];

	dc1=b[18];
	dc2=b[19];
	dc3=b[20];
	dc4=b[21];
	dc5=b[22];

  	ae0=b[23];
	ae1=b[24];
	ae2=b[25];
	ae3=b[26];
	ae4=b[27];
	ae5=b[28];

	be0=b[29];
	be1=b[30];
	be2=b[31];
	be3=b[32];
	be4=b[33];
	be5=b[34];

	ce0=b[35];
	ce1=b[36];
	ce2=b[37];
	ce3=b[38];
	ce4=b[39];
	ce5=b[40];

	de0=b[41];
	de1=b[42];
	de2=b[43];
	de3=b[44];
	de4=b[45];
	de5=b[46];


	fce1=b[47];
	fce2=b[48];
	fce3=b[49];
	fce4=b[50];
	fce5=b[51];

	fec0=b[52];
	fec1=b[53];
	fec2=b[54];
	fec3=b[55];
	fec4=b[56];
	fec5=b[57];


if itype == 0;

	betamc = cdfn(b[58]);
	betame = cdfn(b[59]);
	betaw   = cdfn(b[60]);


	alphamc = cdfn(b[58] + abs(b[61]) ) - betamc;
	alphame = cdfn(b[59] + abs(b[62]) ) - betame; 
	alphaw = cdfn(b[60] + abs(b[63]) ) - betaw; 

	pmc=tanh(b[75]);
	pme=tanh(b[76]);
	pmw=tanh(b[77]);

    else;       /**     for computing standard errors   **/

	betamc = (b[58]);
	betame = (b[59]);
	betaw   = (b[60]);


	alphamc = b[61];
	alphame = b[62]; 
	alphaw  = b[63]; 

	pmc=(b[75]);
	pme=(b[76]);
	pmw=(b[77]);

endif;


	gce1=b[64];
	gce2=b[65];
	gce3=b[66];
	gce4=b[67];
	gce5=b[68];

	gec0=b[69];
	gec1=b[70];
	gec2=b[71];
	gec3=b[72];
	gec4=b[73];
	gec5=b[74];


	dm1=b[78];
	dm2=b[79];
	dm3=b[80];
	dm4=b[81];
	dm5=b[82];

	hmct = recsercp( alphamc*(0.0 | trimr(umct.^2,0,1) ) + betamc , 1-alphamc-betamc);
	hmet = recsercp( alphame*(0.0 | trimr(umet.^2,0,1) ) + betame , 1-alphame-betame);
	hwt = recsercp( alphaw*(0.0 | trimr(uwt.^2,0,1) ) + betaw , 1-alphaw-betaw);

	emct = umct.*sqrt(hmct);	
	emet = umet.*sqrt(hmet);
	ewt   = uwt.*sqrt(hwt);

	mct=recserar(emct,0.0,pmc);
	met=recserar(emet,0.0,pme);
	wt=recserar(ewt,0.0,pmw);

	emct=trimr(emct,1,0);
	emet=trimr(emet,1,0);	

	mctl=trimr(mct,0,1);
	mct=trimr(mct,1,0);
	metl=trimr(met,0,1);
	met=trimr(met,1,0);

/*
	emetl = trimr(emet,0,1);
	emet  = trimr(emet,1,0);

	emctl = trimr(emct,0,1);
	emct  = trimr(emct,1,0);
*/



	ytsim =         ac1*ucc1t + ac0*ucc0t + bc1*mct + cc0*uz0t + cc1*uz1t + dc1*wt  +  fce1*metl + gce1*emet;
	ytsim = ytsim ~ ac2*ucc2t + ac0*ucc0t + bc2*mct + cc0*uz0t + cc2*uz2t + dc2*wt  +  fce2*metl + gce2*emet;
	ytsim = ytsim ~ ac3*ucc3t + ac0*ucc0t + bc3*mct + cc0*uz0t + cc3*uz3t + dc3*wt  +  fce3*metl + gce3*emet ; 
	ytsim = ytsim ~ ac4*ucc4t + ac0*ucc0t + bc4*mct + cc0*uz0t + cc4*uz4t + dc4*wt  +  fce4*metl + gce4*emet; 
	ytsim = ytsim ~ ac5*ucc5t + ac0*ucc0t + bc5*mct + cc0*uz0t + cc5*uz5t + dc5*wt  +  fce5*metl + gce5*emet;

	ytsim = ytsim ~ ae0*uce0t + be0*met + ce0*uz0t + de0*wt +  fec0*mctl + gec0*emct;
	ytsim = ytsim ~ ae1*uce1t + be1*met + ce1*uz1t + de1*wt +  fec1*mctl + gec1*emct + dm1*uce0tlg;
	ytsim = ytsim ~ ae2*uce2t + be2*met + ce2*uz2t + de2*wt +  fec2*mctl + gec2*emct + dm2*uce0tlg;
	ytsim = ytsim ~ ae3*uce3t + be3*met + ce3*uz3t + de3*wt +  fec3*mctl + gec3*emct + dm3*uce0tlg;
	ytsim = ytsim ~ ae4*uce4t + be4*met + ce4*uz4t + de4*wt +  fec4*mctl + gec4*emct + dm4*uce0tlg;
	ytsim = ytsim ~ ae5*uce5t + be5*met + ce5*uz5t + de5*wt +  fec5*mctl + gec5*emct + dm5*uce0tlg;



		@develop the criteria for comparison with the true data@


    yt = ytsim;


	ytfact = yt*loadings ;  

	y = trimr(yt,1,0);
	x = trimr(ytfact,0,1);
	u1 = y - x*bhat1;
	g1 = u1 *~ x;


y0 = (u1-meanc(u1)') *~ (u1-meanc(u1)');		@cov matrix calc as 1^2,1*2,1*3,1*4,2*1,2^2,2*3,2*4.....4^4@
y0   = y0[.,vech( reshape(seqa(1,1,cols(u1)^2),cols(u1),cols(u1)) )] ;		@take the lower diagonal of y2@
g0 = y0 - bhat0';	


	pc1 = ytfact[.,3];
	dpc1 = trimr(pc1,1,0) - trimr(pc1,0,1);

	
	pc2 = ytfact[.,2];
	dpc2 = trimr(pc2,1,0) - trimr(pc2,0,1);

	
	pc3 = ytfact[.,1];
	dpc3 = trimr(pc3,1,0) - trimr(pc3,0,1);


	zt = (dpc1.^2 - meanc(dpc1.^2))/stdc(dpc1.^2);
	z2 = trimr(zt,2,0).*trimr(zt,1,1) ~ trimr(zt,2,0).*trimr(zt,0,2);
	g2 = z2 - bhat2';



	zt = (dpc2.^2 - meanc(dpc2.^2))/stdc(dpc2.^2);
	z2 = trimr(zt,2,0).*trimr(zt,1,1) ~ trimr(zt,2,0).*trimr(zt,0,2);
	g3 = z2 - bhat3';


	zt = (dpc3.^2 - meanc(dpc3.^2))/stdc(dpc3.^2);
	z2 = trimr(zt,2,0).*trimr(zt,1,1) ~ trimr(zt,2,0).*trimr(zt,0,2);
	g4 = z2 - bhat4';

  
    g = trimr( g1 ~ g0 , 2, 0)   ~ g2 ~ g3 ~ g4;




		@make the comparison@

            	dq[1,.] =  meanc( g )';


        retp( meanc(dq) );				@ how do the simulations compare with the actual data - minimise @

    endp;



    proc fobj(b);						@this is for the standard errors@
	local mb;
	mb = mbar(b);
        retp( mb'sigma*mb );			@ weighted sum of the simulations compared with actual data - minimise@
    endp;




/*      Estimate the indirect model using simulated data   */


_opalgr  = 2;
__output = 2;
_opgtol  = 0.001;



bstart=0.01*ones(82,1);

/**     h=10        **/

bstart = {
0.392790702994 
0.696581369417 
2.53754311794 
-0.777426848946 
-0.645804080937 
-0.459468680762 
-0.150497208086 
-0.808196159355 
-0.153925956318 
-0.164854728273 
-0.224447193693 
-0.108576389266 
-1.46185279657 
-1.70714611167 
-0.860946802085 
-0.140348576303 
0.225754497016 
-0.356383156418 
-0.998613075863 
-0.616299773498 
-0.781818825417 
-0.183898123050 
-0.783675650618 
-2.29707212749 
-1.47354095537 
1.52911832642 
-1.62399469488 
-0.206436044382 
-0.152094397856 
-1.65978189442 
0.527956558171 
-0.380656764482 
-0.813077036929 
-0.181255507500 
0.803250457223 
0.172779865613 
-0.178468189297 
0.380386193729 
-0.554091460360 
0.00640504698432 
-0.127431148264 
-0.0276414393133 
1.06297889991 
0.274908132112 
1.34193893052 
-0.0324225922574 
1.27438364386 
1.28382977156 
0.0124808070489 
0.141571764878 
-0.141490389349 
-0.0415052983173 
0.878723458306 
1.03437291506 
1.31436271195 
0.854535328727 
0.0431756841582 
0.0430746731665 
0.518187871603 
-0.107213665121 
-0.0158521225117 
1.62891958596 
1.18873007929 
-0.847651847924 
1.44235707118 
0.390962102812 
1.15567119521 
0.185432617495 
0.198311597538 
0.941506312391 
0.627055592841 
0.582607107595 
0.967985126183 
0.590248215301 
0.00608491941886 
0.511542086562 
0.0502519795919 
0.129153255756 
-0.573134672487 
-0.439013111576 
-0.504103954129 
-0.801250696405 
};




/**  h=100  from Vance**/

bstart = {
0.263714388160 
1.58399402039 
3.14764373553 
-0.182385954785 
-0.0962960010346 
0.0752074515224 
-0.597803765015 
-0.866123529694 
-0.234115782132 
0.0354048681037 
-0.264538251592 
-0.0590502142719 
-0.437786255963 
0.212698046621 
-1.11247687074 
0.721707540352 
0.547493286173 
-0.249765863594 
-1.44530575516 
-0.840463749035 
-0.918929675862 
-0.280500776560 
-0.654719505109 
-1.66466871558 
-0.353476161040 
1.57126449779 
-1.75128201557 
0.177295007545 
-0.304358112140 
-1.59331401276 
0.328591467758 
-0.443657500015 
-0.604630321322 
-0.311278017732 
0.773722556199 
1.77720342985 
1.53753814354 
0.326626792452 
-0.301814218455 
-0.0302955710923 
-0.0969842177546 
0.112604742825 
0.837868731091 
0.308145921363 
0.994685729284 
-0.0310311694006 
0.842937589666 
1.83733605739 
0.102205708842 
0.251409444394 
-0.121509785374 
0.0979684557858 
1.39873760402 
1.43620599346 
1.63100610622 
1.50826010532 
0.228066494767 
0.922628510556 
-0.0131236054775 
1.04637696436 
1.53264484935 
1.61593382283 
0.297157609920 
-0.562856214810 
1.40936889244 
0.217472815383 
0.930896970305 
0.256494545154 
0.380568350515 
0.241810289404 
0.386440262963 
-0.0420192357090 
0.539741424779 
0.338515080716 
0.445258479777 
0.262636551089 
0.0605630869336 
-0.0642317989169 
-0.542711444830 
-0.465081582197 
-0.595109875932 
-0.880136602002 
};




iter = 0;

itype = 0.0;

/*
{b,f,g_deriv,retcode} = optprt(optmum(&fobj,bstart));		@do the indirect estimation@
*/

b=bstart';


qmin = t*fobj(bstart);
nmom  = rows(sigma);
npara = rows(b);
dof = nmom - npara;

print "";
print "";
print "EMM Results";
print "***********";
print "The value of the objective function is " qmin;
if dof > 0.0; print "Overidentifying restriction test p-value is " cdfchic(qmin,dof); 
print "     With degrees of freedom =" dof; endif;




/**     Compute standard errors       **/

    itype = 1.0;

	b[58] = betamc;
	b[59] = betame;
	b[60] = betaw;


	b[61] = alphamc;
	b[62] = alphame; 
	b[63] = alphaw; 

	b[75] = pmc;
	b[76] = pme;
	b[77] = pmw;



    g0   = gradp(&mbar,b);
    vcov = inv(g0'inv(i0)*g0)/t;
    se   = sqrt(diag(vcov));



format /rd 12,3;
print "";
print "   Estimates    std errors       t-stats   " 
b~se~(b./se);


print "";
print "";





/**     Construct parameters     

    delta         World parameters               
    lam           Market parameters              
    omega         Country parameters             
    d             Spillover parameters           
    gam           Contagion parameters 
    phi           Unique in country market parameters 
    kappa         Time zone effect parameters     
**/


@its world, market, country0, country,spillover, contagion, unique-0, unique, timezone@
    delta = b[18:22] |  b[41:46];
    lam   = ( b[7:11] | zeros(6,1) ) ~ ( zeros(5,1) | b[29:34] );
    omega = ( b[12]*ones(5,1) ~ diagrv(eye(5),b[13:17]) ) | diagrv(eye(6),b[35:40]);  
    d     = ( b[47:51] | zeros(6,1) ) ~ ( zeros(5,1) | b[52:57] );
    gam   = ( b[64:68] | zeros(6,1) ) ~ (zeros(5,1) | b[69:74]);
    phi   = ( b[1]*ones(5,1) | zeros(6,1) ) ~ diagrv(eye(11),b[2:6]|b[23:28]);
    kappa = zeros(6,1) | b[78:82];


/**     Adjust parameters for long-run calculations     **/

@check Vance's old stuff@
    olddelta=delta;
    oldd=d;

    delta = delta./sqrt(1-pmw^2);       /** This new  **/
    lam   = lam./sqrt(1 - (pmc~pme).^2);
    d     = d./sqrt(1 - (pmc~pme).^2);  /** This is new  **/

    /**     Combine matrices and compute long-run decomposition       **/

    decomp = (delta ~ lam ~ omega ~ d ~ gam ~ phi ~ kappa).^2;

    total  = sumc(decomp');

    decomp = 100*decomp./total;  
 

/*
    decomp = 100*total.^2 ./ sumc( (total.^2)');
*/

rc=zeros(8,1);
for k(1,5,1);
    r1=decomp[k,.]';
    r2=selif(r1,r1.gt 0);
    rc=rc~r2;
endfor;
rc=rc[.,2:cols(rc)];

re=decomp[6,.]';
re=selif(re,re.gt 0);
re=re|0;
for k(7,11,1);
    r1=decomp[k,.]';
    r2=selif(r1,r1.gt 0);
    re=re~r2;
endfor;


/**         Compute standard errors of variance decompositions      **/

proc dc(b);

    local pmc,pme,pmw,delta,lam,omega,d,gam,phi,kappa,total,decomp;

	pmc = (b[75]);
	pme = (b[76]);
	pmw = (b[77]);

    delta = b[18:22] |  b[41:46];
    lam   = ( b[7:11] | zeros(6,1) ) ~ ( zeros(5,1) | b[29:34] );
    omega = ( b[12]*ones(5,1) ~ diagrv(eye(5),b[13:17]) ) | diagrv(eye(6),b[35:40]);  
    d     = ( b[47:51] | zeros(6,1) ) ~ ( zeros(5,1) | b[52:57] );
    gam   = ( b[64:68] | zeros(6,1) ) ~ (zeros(5,1) | b[69:74]);
    phi   = ( b[1]*ones(5,1) | zeros(6,1) ) ~ diagrv(eye(11),b[2:6]|b[23:28]);
    kappa = zeros(6,1) | b[78:82];

    delta = delta./sqrt(1-pmw^2);       /** This new  **/
    lam   = lam./sqrt(1 - (pmc~pme).^2);
    d     = d./sqrt(1 - (pmc~pme).^2);  /** This is new  **/

    total = delta ~ lam ~ omega ~ d ~ gam ~ phi ~ kappa;  

/*
    decomp = 100*total.^2 ./ sumc( (total.^2)');

decomp = delta[1]^2;
*/

    decomp = total.^2;

    retp( vec(decomp) );

endp;


v2 = dc(b);

v2 = reshape(v2,26,11)';

totalv2 = sumc(v2');

v2 = 100*v2./totalv2;


dv0 = gradp(&dc,b);

sev2  = sqrt( diag( dv0*vcov*dv0') ) ;   /**    Variance decomp standard errors  **/

sev2 = reshape(sev2,26,11)';

sev2 = 100*sev2./totalv2;               /**     Scale standard errors by totalv2    **/


tmp = selif(vec(v2)~vec(sev2),vec(v2) .gt 0.0);


print "Variance decomposition (%) and scaled standard errors";
print "     Estimate  Standard error";
tmp;

print "";
print "";

stop;


format /rdt 2,3;

dlab="Korea "$|"Indon "$|"Thai "$|"Malay "$|"Aust ";
print "Currency markets"; 
dlab';
rc;
print "";

dlab="US"$|"Korea "$|"Indon "$|"Thai "$|"Malay "$|"Aust ";
print "Equity markets"; 
dlab';
re;






stop;



/********************

h=100


EMM Results
***********
The value of the objective function is     16.11632 
Overidentifying restriction test p-value is      0.85024 
     With degrees of freedom =    23.00000 

   Estimates    std errors       t-stats   
       0.264        0.163        1.615 
       1.584        0.468        3.383 
       3.148        0.511        6.155 
      -0.182        6.646       -0.027 
      -0.096        5.013       -0.019 
       0.075        2.436        0.031 
      -0.598        1.598       -0.374 
      -0.866        2.260       -0.383 
      -0.234        0.721       -0.325 
       0.035        0.259        0.137 
      -0.265        0.764       -0.346 
      -0.059        0.115       -0.516 
      -0.438        1.464       -0.299 
       0.213        0.645        0.330 
      -1.112        1.065       -1.044 
       0.722        0.744        0.970 
       0.547        0.331        1.653 
      -0.250        0.282       -0.886 
      -1.445        0.801       -1.805 
      -0.840        0.395       -2.127 
      -0.919        0.518       -1.775 
      -0.281        0.212       -1.324 
      -0.655        0.562       -1.166 
      -1.665        6.874       -0.242 
      -0.353        8.248       -0.043 
       1.571        0.185        8.496 
      -1.751        0.175      -10.032 
       0.177        2.666        0.067 
      -0.304        1.815       -0.168 
      -1.593       10.091       -0.158 
       0.329        2.059        0.160 
      -0.444        2.861       -0.155 
      -0.605        3.793       -0.159 
      -0.311        1.894       -0.164 
       0.774        0.481        1.607 
       1.777        6.141        0.289 
       1.538        1.959        0.785 
       0.327        0.397        0.822 
      -0.302        0.452       -0.667 
      -0.030        0.123       -0.247 
      -0.097        0.117       -0.826 
       0.113        0.374        0.301 
       0.838        0.430        1.950 
       0.308        0.293        1.053 
       0.995        0.609        1.632 
      -0.031        0.129       -0.241 
       0.843        5.462        0.154 
       1.837       11.202        0.164 
       0.102        0.652        0.157 
       0.251        1.394        0.180 
      -0.122        0.794       -0.153 
       0.098        0.322        0.304 
       1.399        3.706        0.377 
       1.436        3.609        0.398 
       1.631        4.189        0.389 
       1.508        3.790        0.398 
       0.228        0.538        0.424 
       0.822        1.343        0.612 
       0.495        2.894        0.171 
       0.852       12.195        0.070 
       0.171        1.199        0.143 
       0.451        1.064        0.424 
       0.058        1.926        0.030 
      -0.563        3.493       -0.161 
       1.409        9.298        0.152 
       0.217        1.523        0.143 
       0.931        5.962        0.156 
       0.256        1.587        0.162 
       0.381        0.950        0.400 
       0.242        1.065        0.227 
       0.386        1.626        0.238 
      -0.042        0.785       -0.054 
       0.540        2.004        0.269 
       0.339        0.845        0.401 
       0.418        0.320        1.305 
       0.257        0.442        0.581 
       0.060        0.192        0.315 
      -0.064        0.309       -0.208 
      -0.543        0.392       -1.383 
      -0.465        0.387       -1.201 
      -0.595        0.458       -1.300 
      -0.880        0.512       -1.719 


Variance decomposition (%) and standard errors
     Estimate  Standard error

       1.408        6.011 
      10.972       48.264 
      32.533       25.844 
      35.382      181.411 
      12.617       29.963 
       0.731        1.787 
       0.118        1.054 
      11.820       26.006 
       1.602        5.204 
      13.026       28.852 
       0.089        0.700 
       9.737       52.221 
       4.757       25.084 
       3.048       17.516 
       0.063        1.049 
      13.548       65.316 
       7.682       82.088 
      25.125      230.482 
       1.939       22.974 
       3.543       42.320 
       5.134       58.238 
       9.587      101.494 
       0.078        0.391 
       0.018        0.097 
       0.160        0.599 
       0.146        0.860 
       0.557        2.301 
      46.370       86.210 
       4.310       36.884 
      29.198      189.667 
       0.237        1.896 
      39.656      134.166 
      56.791      110.098 
       1.794        6.592 
      21.744      110.935 
       1.195        4.226 
      47.892       99.925 
       0.085        0.669 
      19.360      182.441 
      21.407      173.644 
       0.581        7.085 
       3.197       19.529 
       2.858       32.022 
       0.796        4.744 
      19.363       88.085 
      37.043      107.429 
      47.884      123.948 
      31.947       97.074 
       5.147       20.689 
       7.124       64.510 
      10.395       96.135 
       2.170       29.518 
      36.177      284.794 
      10.511      110.662 
      11.218       47.720 
       0.541        3.813 
       2.505       16.341 
       0.030        1.130 
       3.821       20.994 
      10.591       42.015 
       1.564        5.716 
       0.364        1.521 
       3.191        4.542 
       2.903       14.814 
      11.112       26.610 
      56.419      188.563 
      51.850      206.985 
       1.526      111.327 
       0.387       40.614 
       0.904       58.965 
      33.203       64.379 
      25.618      258.793 
       2.096       97.761 
      41.511      108.987 
      40.231       88.407 
       2.905       87.085 
       0.038        0.369 
       4.941        8.867 
       3.637        8.148 
       4.646        8.442 
      71.595      142.438 


Currency markets
          Korea            Indon             Thai            Malay             Aust  

1.408	10.972	32.533	35.382	12.617	
9.737	4.757	3.048	0.063	13.548	
0.078	0.018	0.160	0.146	0.557	
4.310	0.237	56.791	21.744	47.892	
19.360	21.407	0.581	3.197	2.858	
7.124	10.395	2.170	36.177	10.511	
1.564	0.364	3.191	2.903	11.112	
56.419	51.850	1.526	0.387	0.904	

Equity markets
              US           Korea            Indon             Thai            Malay             Aust  

0.731	0.118	11.820	1.602	13.026	0.089	
7.682	25.125	1.939	3.543	5.134	9.587	
46.370	29.198	39.656	1.794	1.195	0.085	
0.796	19.363	37.043	47.884	31.947	5.147	
11.218	0.541	2.505	0.030	3.821	10.591	
33.203	25.618	2.096	41.511	40.231	2.905	
0.000	0.038	4.941	3.637	4.646	71.595	

****/